#include "WPILib.h"
#include "Bucket.h"
#include "IOMap.h"

const double kBeaterSpeed = .5;
const double kIndexerSpeedBlock = .5;
const double kIndexerSpeedIndex = 1.0;
const double kConveySpeed = 0.5;

Bucket::Bucket():
	conveyor(CAN_ID_CONVEYOR),
	beater(CAN_ID_INTAKE_SECONDARY),
	indexer(CAN_ID_CONVEYOR_INDEXER),
	encoder(DIG_IN_TRANSITION_ENCODER),
	_indexerSpeedBlock(kIndexerSpeedBlock),
	_indexerSpeedIndex(kIndexerSpeedIndex),
	_conveyDirection(OFF),
	_beaterDirection(STOP),
	_indexDirection(STOP),
	_useSensor(true),
	_conveySpeed(kConveySpeed),
	_beaterSpeed(kBeaterSpeed)
	{
	}


	void Bucket::Start_Enqueue(ConveyorDirection direction)
	{
		_conveyDirection = direction;
	}

	void Bucket::Stop_Enqueue()
	{
		_conveyDirection = OFF;
	}

	void Bucket::Beating_Bar_Set_Direction(BeaterDirection bdirection)
	{
		_beaterDirection = bdirection;

	}

	void Bucket::Set_Sensor_Broken(bool broken){
		_useSensor = broken;
	}

	void Bucket::Process()
	{
		if(_conveyDirection == OFF)
		{
			conveyor.Set(0);
			switch(_beaterDirection){
				case IN:
					beater.Set(_beaterSpeed);
				break;
				case OUT:
					beater.Set(-_beaterSpeed);
				break;
				case STOP:
					beater.Set(0);
				break;
			}
		}
		else if(_conveyDirection == FORWARD)
		{
			beater.Set(_beaterSpeed);
			conveyor.Set(_conveySpeed);

		}
		else if(_conveyDirection == BACKWARD)
		{
			beater.Set(-_beaterSpeed);
			conveyor.Set(-1 * _conveySpeed);
		}
		switch(_indexDirection)
		{
		case IN:
			indexer.Set(_indexerSpeedIndex);
		break;
		case OUT:
			indexer.Set(-_indexerSpeedBlock);
		break;
		case STOP:
			indexer.Set(0);
		break;
		}

	}

	void Bucket::Reset()
	{
		beater.Set(0);
		conveyor.Set(0);
		_conveyDirection = OFF;
		_beaterDirection = STOP;
		encoder.Reset();
	}

	void Bucket::Init_Dashboard_Values(Feedback *feedback) {
		feedback->send_Editable_Double("bucket", "beater speed", _beaterSpeed);
		feedback->send_Editable_Double("bucket", "indexer speed block", _indexerSpeedBlock);
		feedback->send_Editable_Double("bucket", "indexer speed index", _indexerSpeedIndex);
		feedback->send_Editable_Double("bucket", "convey speed", _conveySpeed);
	}

	void Bucket::Get_Dashboard_Values(Feedback *feedback){
		_beaterSpeed = feedback->get_Editable_Double("bucket", "beater speed", _beaterSpeed);
		_indexerSpeedBlock = feedback->get_Editable_Double("bucket", "indexer speed block", _indexerSpeedBlock);
		_indexerSpeedIndex = feedback->get_Editable_Double("bucket", "indexer speed index", _indexerSpeedIndex);
		_conveySpeed = feedback->get_Editable_Double("bucket", "convey speed", _conveySpeed);
	}

	void Bucket::Debug(Feedback *feedback)
	{
		feedback->send_Debug_Double("bucket", "Encoder Count", encoder.Get());
		feedback->send_Debug_Double("bucket", "Beater Direction", _beaterDirection);
		feedback->send_Debug_Double("bucket", "Convey Direction", _conveyDirection);
		feedback->send_Debug_Double("bucket", "Index Direction", _indexDirection);
		feedback->send_Debug_Double("bucket", "Index Current Draw", indexer.GetOutputCurrent());
		feedback->send_Debug_Double("bucket", "Convey Current Draw", conveyor.GetOutputCurrent());
		feedback->send_Debug_Double("bucket", "Beater Current Draw", beater.GetOutputCurrent());
	}


	void Bucket::Indexer_Set_Direction(BeaterDirection direction)
	{
		_indexDirection = direction;
	}

