#include "WPIlib.h"
#include "Climbing.h"
#include "IOMap.h"

const double kClimbSpeedFast = .85;
const double kServoPos = .3;
const double kStopCurrent = 80;


//Array to convert our enum into a readable string

	Climbing::Climbing() :
			_climbFinished(false),
			MotorA (CAN_ID_CLIMBER),
			MotorB(CAN_ID_CLIMBER2),
			Direction (IN),
			DeployServo (PWM_OUT_SCALER_DEPLOY_SERVO),
			_currentValues(),
			_stallIdx(0),
			_servoPos(kServoPos),
			_climberSpeed(kClimbSpeedFast),
			_stopCurrent(kStopCurrent)
	{
		MotorB.SetControlMode(MotorB.kFollower);
		MotorB.Set(CAN_ID_CLIMBER);


	}

	void Climbing::Climb_Direction(ClimbingDirection direction, double speed){
		if(!Is_Climb_Complete()){
			Direction=direction;
			_climberSpeed = speed;
		}
	}

	bool Climbing::Is_Climb_Complete(){
		return _climbFinished;
	}


	void Climbing::Process(){
		if(!_climbFinished){
					float avg = 0;
					_currentValues[_stallIdx] = MotorA.GetOutputCurrent();		//sets a value in the array to the current output
					if(_stallIdx < STALLIDXNUM){				//if the number in the array isn't the last one...
						_stallIdx++;					//make it the next number
					}
					else{								//if it is the last one...
							_stallIdx = 0;					//start over
					}
					for(int i=0; i<STALLIDXNUM; i++)
					{			//adds up all the numbers in the array
						avg =+ _currentValues[i];
					}
					avg = avg/STALLIDXNUM;					//averages all the numbers

					if(avg > _stopCurrent)			//if the average is more than the stop limit...
					{
						Direction = STOP;
						_climbFinished = true;
						SmartDashboard::PutNumber("thunderdashboard_gyro", 1);
					}
					else
					{
							_climbFinished = false;

					}
		}

		if (Direction==IN){
				MotorA.Set(_climberSpeed);

		}

		if(Direction==OUT){
			MotorA.Set (-_climberSpeed);
		}
		if (Direction==STOP){
			MotorA.Set(0);
		}
	}

	void Climbing::Reset(){
		Direction = STOP;
		DeployServo.SetOffline();
		MotorA.Set(0);
		_climbFinished = false;

	}
	void Climbing::Debug(Feedback *feedback){
		feedback->send_Debug_Double("climber", "Direction", Direction);
		feedback->send_Debug_Double("climber", "Master Current", MotorA.GetOutputCurrent());
		feedback->send_Debug_Double("climber", "Follower Current", MotorB.GetOutputCurrent());
		feedback->send_Debug_String("climber", "finished", _climbFinished ? "true":"false");

	}

	void Climbing::Deploy(){
		DeployServo.Set(_servoPos); //value to be determined
	}

	void Climbing::Init_Dashboard_Values(Feedback *feedback){
		feedback->send_Editable_Double("climber", "stop current", _stopCurrent);
		feedback->send_Editable_Double("climber", "speed", _climberSpeed);
		feedback->send_Editable_Double("climber", "servo", _servoPos);
	}

	void Climbing::Get_Dashboard_Values(Feedback *feedback){
		_stopCurrent = feedback->get_Editable_Double("climber", "stop current", _stopCurrent);
		_climberSpeed = feedback->get_Editable_Double("climber", "speed", _climberSpeed);
		_servoPos = feedback->get_Editable_Double("climber", "servo", _servoPos);
	}
