#include "WPILib.h"
#include "Controls.h"
#include <GenericHID.h>


#define JOYSTICK_DRIVER 0
#define DRIVER_LEFT_Y_AXIS 1			// up/down
#define DRIVER_RIGHT_X_AXIS 4     // left/right
#define DRIVER_RIGHT_Y_AXIS 5
#define DRIVER_LEFT_SLOW_BUTTON 6
#define DRIVER_RIGHT_SLOW_BUTTON 5
#define DRIVER_LEFT_TRIGGER_AXIS 3
#define DRIVER_RIGHT_TRIGGER_AXIS 2
#define DRIVER_SWAP_DRIVE_BUTTON 1
#define DRIVER_SWAP_CAMERAS_BUTTON 4


#define JOYSTICK_AUX 1
#define AUX_SHOOT_BUTTON 1
#define AUX_CALIBRATE_GYRO 4
#define AUX_PREHEAT_BUTTON 3
#define AUX_RBUMPER_INTAKE 6
#define AUX_LBUMPER_CLOSE_GEAR 5
#define AUX_CLIMB_DPAD_UP 0
#define AUX_OUTTAKE_TRIGGER 3
#define AUX_RELEASE_GEAR_TRIGGER 2
#define AUX_CLIMBER_RESET 2

#define JOYSTICK_BROKEN 2
#define SHOOTER_ENCODER_BROKEN 1
#define GEAR_SPIKE_SENSOR_BROKEN 2
#define BUCKET_CONVERYOR_ENCODER_BROKEN 3
#define DRIVE_LEFT_ENCODER_BROKEN 4
#define DRIVE_RIGHT_ENCODER_BROKEN 5
#define DRIVE_GYRO_BROKEN 6

Controls::Controls(Drive *drive, Gear *gear, FuelSystem *fuelsystem, Feedback *feedback, Climbing *climbing):
	timer()
{
	// save given object pointers, for later use
	_fuelsystem =fuelsystem;
	_drive = drive;
	_feedback = feedback;
	_gear = gear;
	_climbing = climbing;



	_driverJoystick = new Joystick(JOYSTICK_DRIVER);
	_driverSwapDrive = new ControlsButton(_driverJoystick, DRIVER_SWAP_DRIVE_BUTTON);
	_swapDrive = false;

	_auxJoystick = new Joystick(JOYSTICK_AUX);
	_auxClimb = new ControlsButton(_auxJoystick, AUX_CLIMB_DPAD_UP, true);
	_spikeRumble = false;

	DashboardTime.Start();

	_brokenJoystick = new Joystick(JOYSTICK_BROKEN);
	_brokenShooter = new ControlsButton(_brokenJoystick, SHOOTER_ENCODER_BROKEN);
	_brokenSpikeSensor = new ControlsButton(_brokenJoystick, GEAR_SPIKE_SENSOR_BROKEN);
	_brokenConveyorEncoder = new  ControlsButton(_brokenJoystick, BUCKET_CONVERYOR_ENCODER_BROKEN);
	_brokenLeftEncoder = new ControlsButton(_brokenJoystick, DRIVE_LEFT_ENCODER_BROKEN);
	_brokenRightEncoder = new ControlsButton(_brokenJoystick, DRIVE_RIGHT_ENCODER_BROKEN );
	_brokenGyro = new ControlsButton(_brokenJoystick, DRIVE_GYRO_BROKEN );

	/*
	_auxShoot = new ControlsButton(_auxJoystick, AUX_SHOOT_BUTTON);
	_auxPreHeat = new ControlsButton(_auxJoystick, AUX_PREHEAT_BUTTON);
	_auxIntake = new ControlsButton(_auxJoystick, AUX_RBUMPER_INTAKE);
	_auxCloseGear = new ControlsButton(_auxJoystick, AUX_LBUMPER_CLOSE_GEAR);
	_auxClimb = new ControlsButton(_auxJoystick, AUX_CLIMB_DPAD_UP, true);
	*/
}

Controls::~Controls()
{
}
void Controls::Process(RobotMode robotmode)
{
	ProcessControllerDriver();
	ProcessControllerAux();
	ProcessBroken();
	DashboardValues();
}
void Controls::ProcessControllerDriver()
{
		double driveLeftY;
		double driveRightX;
		bool slowLeft;
		bool slowRight;
		bool turboLeft;
		bool turboRight;
		bool buttonCamSwap;
		double motorDriveLeft;
		double motorDriveRight;

		if(!_swapDrive){
		driveLeftY = -GetPosition(_driverJoystick, DRIVER_LEFT_Y_AXIS); // joystick Y values are inverted
		driveRightX = GetPosition(_driverJoystick, DRIVER_RIGHT_X_AXIS);
		turboLeft = (GetPosition(_driverJoystick, DRIVER_LEFT_TRIGGER_AXIS) > 0);
		turboRight = (GetPosition(_driverJoystick, DRIVER_RIGHT_TRIGGER_AXIS) > 0);
		slowLeft = _driverJoystick->GetRawButton(DRIVER_LEFT_SLOW_BUTTON);
		slowRight = _driverJoystick->GetRawButton(DRIVER_RIGHT_SLOW_BUTTON);
		}
		else{
			driveLeftY = GetPosition(_driverJoystick, DRIVER_LEFT_Y_AXIS); // joystick Y values are inverted
			if(driveLeftY < .05 && driveLeftY > -.05)
				driveRightX = GetPosition(_driverJoystick, DRIVER_RIGHT_X_AXIS);
			else
				driveRightX = -GetPosition(_driverJoystick, DRIVER_RIGHT_X_AXIS);
			turboRight = (GetPosition(_driverJoystick, DRIVER_LEFT_TRIGGER_AXIS) > 0);
			turboLeft = (GetPosition(_driverJoystick, DRIVER_RIGHT_TRIGGER_AXIS) > 0);
			slowRight = _driverJoystick->GetRawButton(DRIVER_LEFT_SLOW_BUTTON);
			slowLeft =  _driverJoystick->GetRawButton(DRIVER_RIGHT_SLOW_BUTTON);
		}
		buttonCamSwap = _driverJoystick->GetRawButton(DRIVER_SWAP_CAMERAS_BUTTON);

		if (driveRightX == 0)
		{

			// drive forward and back
			motorDriveLeft = GetPower(driveLeftY, slowLeft, turboLeft);
			motorDriveRight = GetPower(driveLeftY, slowRight, turboRight);
		}
		else if (driveLeftY == 0)
		{
			// spins left and right
			motorDriveLeft = GetPowerTurn(driveRightX, slowLeft, turboLeft);
			motorDriveRight = -GetPowerTurn(driveRightX, slowRight, turboRight);
		}
		else
		{
			// arching - so set both to same speed (for now)
			motorDriveLeft = GetPower(driveLeftY, slowLeft, turboLeft);
			motorDriveRight = GetPower(driveLeftY, slowRight, turboRight);
			//printf("DLY %f \n", driveLeftY);
			//printf("DRX %f \n", driveRightX);
			// decrease one side, scale it by how much the right stick is moved
			if (driveRightX > 0)
			{
				// arcing right (decrease right motor)
				motorDriveRight *= (1-(driveRightX * .75));
			}
			else
			{
				// arcing left (decrease left motor)
				// NOTE: invert joystick so we have a positive value for the scale factor
				motorDriveLeft *= -(-1-(driveRightX * .75));
			}
		}
		// see if drive needs to be swapped

		if ((_driverSwapDrive->Process() && _driverSwapDrive->Pressed()))
		{
			_swapDrive = !_swapDrive;
			/*
			 * 	   /\
			 *    /  \
			 *   /    \
			 *  /      \
			 * /        \
			if (_swapDrive == false)
			{
				_swapDrive = true;
			}
			else
			{
				_swapDrive = false;
			}  */
		}


		if(buttonCamSwap){
			_feedback->swapCamera();
		}

			// X if drive needs to be swapped, then just negate the power X
			// The above is less then true, you also have to change which side is getting wich values, as "left and right"
			// is objective to w
			if (_swapDrive == true)
			{
			/*	motorDriveLeft = -motorDriveRight;
				motorDriveRight = -motorDriveLeft;
				_feedback->send_Debug_Double("thunderdashboard", "drivereverse", 1);
				*/
			}
			else {
				_feedback->send_Debug_Double("thunderdashboard", "drivereverse", 0);
			}

			_drive->Move_Speed(motorDriveLeft, motorDriveRight);
}
void Controls::ProcessControllerAux()
{
	double climbing;
	bool buttonShoot;
	bool buttonPreheat;
	bool buttonCalibrate;
	bool buttonClimbReset;
	bool buttonIntake;
	//bool buttonCloseGear;
//	bool buttonClimb;
	float triggerOuttake;
	float triggerReleaseGear;

	_auxClimb->Process();

	buttonShoot = _auxJoystick->GetRawButton(AUX_SHOOT_BUTTON);
	buttonPreheat = _auxJoystick->GetRawButton(AUX_PREHEAT_BUTTON);
	buttonCalibrate = _auxJoystick->GetRawButton(AUX_CALIBRATE_GYRO);
	buttonClimbReset = _auxJoystick->GetRawButton(AUX_CLIMBER_RESET);
	buttonIntake = _auxJoystick->GetRawButton(AUX_RBUMPER_INTAKE);
	//buttonCloseGear = _auxJoystick->GetRawButton(AUX_LBUMPER_CLOSE_GEAR);
//	buttonClimb = _auxClimb->Pressed();
	climbing = -_auxJoystick->GetRawAxis(DRIVER_LEFT_Y_AXIS);
	triggerOuttake = (GetPosition(_auxJoystick, AUX_OUTTAKE_TRIGGER) > 0);
	triggerReleaseGear = (GetPosition(_auxJoystick, AUX_RELEASE_GEAR_TRIGGER) > 0);

	if(buttonShoot)
	{
		_fuelsystem->SetAction(FuelSystem::SHOOT);
	} else if (buttonPreheat){
		if(buttonIntake)
		{
			_fuelsystem->SetAction(FuelSystem::INTAKE_PREHEAT);
		} else {
			_fuelsystem->SetAction(FuelSystem::PREHEAT);
		}
	}
	else if(buttonIntake)
	{
		_fuelsystem->SetAction(FuelSystem::INTAKE);
	}else if(triggerOuttake>0)
	{
		_fuelsystem->SetAction(FuelSystem::OUTAKE);
	}
	else
	{
		_fuelsystem->SetAction(FuelSystem::STOP);
	}
	if(triggerReleaseGear>0)	{
		_gear->Extend_Flippers();
	}
	else //if(buttonCloseGear)
	{
		_gear->Retract_Flippers();
		if(GearRumble()){
			_auxJoystick->SetRumble(Joystick::kLeftRumble,0.0);
			_auxJoystick->SetRumble(Joystick::kRightRumble,0.0);
		}
		else{
		//	_auxJoystick->SetRumble(Joystick::kLeftRumble,.5);
		//	_auxJoystick->SetRumble(Joystick::kRightRumble,.5);
		}
	}
	if(climbing > .2)
	{
		_climbing->Climb_Direction(Climbing::IN, climbing);
	}
	else
	{
		_climbing->Climb_Direction(Climbing::STOP, 0);
	}
	if(buttonCalibrate){
		_drive->Calibrate_Gyro();
	}
	if(buttonClimbReset){
		_climbing->_climbFinished = false;
		SmartDashboard::PutNumber("thunderdashboard_gyro", 0);
	}

}
void Controls::ProcessBroken(){
	_brokenShooter->Process();
	_brokenSpikeSensor->Process();
	_brokenConveyorEncoder->Process();
	_brokenLeftEncoder->Process();
	_brokenRightEncoder->Process();
	_brokenGyro->Process();

	_fuelsystem->Set_Shooter_Broken(_brokenShooter->Pressed());
	_gear->Set_Spike_Sensor_Broken(_brokenSpikeSensor->Pressed());
	_fuelsystem->Set_Bucket_Broken(_brokenConveyorEncoder->Pressed());
	_drive->Left_Encoder_Set_Broken(_brokenLeftEncoder->Pressed());
	_drive->Right_Encoder_Set_Broken(_brokenRightEncoder->Pressed());
	_drive->Gyro_Set_Broken(_brokenGyro->Pressed());

}

float Controls::GetPower(float power, bool slow, bool turbo)
{
	// Cube the power to ramp input
		power = power * power * power;

		// Cut power to 3/10 for slower driving
		if (slow)
		{
			power *= .3;
		}
		else if (turbo)
		{
			// Do nothing, keep full power
		}
		else
		{
			// Cut power to 8/10 for normal operation
			power *= .8;
		}
		return (power);
}

float Controls::GetPosition(Joystick *joystick, int axis, bool fullrange)
{
	float position;

	position = joystick->GetRawAxis(axis);

	// Deadzone, prevents motors from running
	if ((position > -.3) && (position < .3))
	{
		position = 0; //when not touching the joysticks
	}
	else if (fullrange){
		if(position > 0){
			position = (position - .3)/.7;
		}
		else{
			position = ((abs(position)-.3)/.7)*-1;
		}
	}
	return (position);
}
float Controls::GetPowerTurn(float power, bool slow, bool turbo)
{
	// Cube the power to ramp input
	power = power * power * power;

	// Cut power to /10 for slower driving
	if (slow)
	{

		power *= .45;
	}
	else if (turbo)
	{
		// Do nothing, keep full power

	}
	else
	{
		// Cut power to 8/10 for normal operation
		power *= .6;
	}
	return (power);
}
bool Controls::GearRumble(){
	if(_gear->Is_Spike_Engaged()){
			if (_spikeRumble == false)
			{
				_spikeRumble = true;
				timer.Reset();
			}
			if(timer.Get() < 2){
				return true;
			}
			else {
				return false;
			}
		}
	else if(!_gear->Is_Spike_Engaged() && _spikeRumble == true){
			_spikeRumble = false;
			return false;
	}
	else{
		return false;
	}
}
void Controls::DashboardValues(){
	if(DashboardTime.Get() > 2){

		_climbing->Get_Dashboard_Values(_feedback);
		_gear->Get_Dashboard_Values(_feedback);
		_fuelsystem->Get_Dashboard_Values(_feedback);
		DashboardTime.Reset();
	}
}
