#ifndef DRIVE_H_
#define DRIVE_H_

#include "Feedback.h"
#include "WPILib.h"
#include <Jaguar.h>


class Drive{
	public:
	Drive();

	void Move_Speed(double leftSpeed, double rightSpeed);
	//called when want to move robot
	//sets left and right motors to left and right speed

	void Auto_Straight(double speed, double distance);
	//moves at a specific speed
	//uses encoders for distance, in inches
	//negative speed to go backwards

	void Auto_Pivot_Turn(float angle);
	//uses gyro to pivot in place to a specific angle
	//negative angle to go left, positive to go right

	void Auto_Vision_Turn(bool left);
	//turns left or right based on input
	//stops turning when vision thing says we straight on
	//uses the auto turn finished

	void Auto_Vision_Straight();
	//drives until stripes out of view
	//uses auto straight finished

	bool Is_Auto_Straight_Finished();
	//returns if straight is finished
	bool Is_Auto_Turn_Finished();
	//returns if turn is finished
	void Debug(Feedback *feedback);

	void Process();

	int Get_Right_Encoder();

	int Get_Left_Encoder();

	void Left_Encoder_Set_Broken(bool broken);

	void Right_Encoder_Set_Broken(bool broken);
	void Gyro_Set_Broken(bool broken);

	void Reset();

	void Calibrate_Gyro();

	bool Did_Auto_Die();

	private:

	void Set_Left_Motors(double Power);
	void Set_Right_Motors(double Power);
	bool CheckTarget(double speed, int distance);	// returns true if on target; will set _leftMotorVision and _rightMotorVision

	typedef enum {
		kTeleop,
		kAutoStraight,
		kAutoTurn,
		kAutoVisionTurn,
		kAutoVisionStraight
	} process;

		SpeedController* leftMini;
		SpeedController* leftMotor;
		SpeedController* rightMini;
		SpeedController* rightMotor;
		Encoder leftEncoder;
		Encoder rightEncoder;
		AnalogGyro gyro;
		double _leftSpeed;
		double _rightSpeed;
		float _autoDistance;
		process drive;
		float _distance;
		float _angle;
		bool _autoTurnFinished;
		bool _autoStraightFinished;
		double _startGyro;
		double _autoSpeed;
		double _leftFinalSpeed;
		double _rightFinalSpeed;
		float _rampTimes;
		double _wantSpeed;
		double _mathAngle;
		bool _rightEncBroke;
		bool _leftEncBroke;
		bool _gyroBroke;
		bool _gyroCalibrated;
		double _currentDriveLeft;
		double _currentDriveRight;

		int _horizontalOffset;
		int _distanceBetween;
		Timer _gyroAnnoyanceTimer;
		bool _annoyanceVal;

		bool _startVisionTracker;
		int _visionCounter;
		int _visionResult;
		int _lastVisionCounter;
		bool _visionCountChanged;
		bool _lastStartVision;
		bool _haveSeen;
		Timer timer;

		double _leftMotorVision = 0;
		double _rightMotorVision = 0;

		int _visionOffsetTolerance;
		int _lostItCounter;





};

#endif
