#include "Feedback.h"
#include "Wpilib.h"
#include <stdio.h>
#include <stdarg.h>
#include <vector>
#include <string>

#define USE_TWO_CAMERAS


static bool camTwoIsPresent()
{
	static const uInt32 MAX_CAMS = 10;
	IMAQdxCameraInformation camInfos[MAX_CAMS];
	uInt32 camCount = MAX_CAMS;
	IMAQdxError err = IMAQdxEnumerateCameras(camInfos, &camCount, 1);
	if (err != IMAQdxErrorSuccess)
		return false;
	for (uInt32 i = 0; i < camCount; ++i) {
		printf("Found camera %s\n", camInfos[i].InterfaceName);
		if (strcmp(camInfos[i].InterfaceName, "cam1") == 0)
			return true;
	}
	return false;
}

static bool camOneIsPresent()
{
	static const uInt32 MAX_CAMS = 10;
	IMAQdxCameraInformation camInfos[MAX_CAMS];
	uInt32 camCount = MAX_CAMS;
	IMAQdxError err = IMAQdxEnumerateCameras(camInfos, &camCount, 1);
	if (err != IMAQdxErrorSuccess)
		return false;
	for (uInt32 i = 0; i < camCount; ++i) {
		printf("Found camera %s\n", camInfos[i].InterfaceName);
		if (strcmp(camInfos[i].InterfaceName, "cam0") == 0)
			return true;
	}
	return false;
}


Feedback::Feedback() : camServer(NULL), frontCam(true), hasBackCam(false), camBrightness(50),
					   camExposure(CAM_AUTO), camWhiteBalance(0)
{
	camServer = ThunderCameraServer::GetInstance();

	strcpy(wbSettings[0].name, "auto");
	wbSettings[0].value = CAM_AUTO;
	strcpy(wbSettings[1].name, "fi");
	wbSettings[1].value = nivision::whiteBalance::kFixedIndoor;
	strcpy(wbSettings[2].name, "fo1");
	wbSettings[2].value = nivision::whiteBalance::kFixedOutdoor1;
	strcpy(wbSettings[3].name, "fo2");
	wbSettings[3].value = nivision::whiteBalance::kFixedOutdoor2;
	strcpy(wbSettings[4].name, "ff1");
	wbSettings[4].value = nivision::whiteBalance::kFixedFluorescent1;
	strcpy(wbSettings[5].name, "ff2");
	wbSettings[5].value = nivision::whiteBalance::kFixedFlourescent2;

	curWbSetting = wbSettings + 4;
	camWhiteBalance = curWbSetting->value;
}

void Feedback::initCameras() {
		std::vector<std::string> v;
		if (camOneIsPresent()){
			v.push_back("cam0");
		}
#ifdef USE_TWO_CAMERAS
		hasBackCam = camTwoIsPresent();
		if (hasBackCam)
			v.push_back("cam1");
#endif
#ifndef USE_USBCAMERACLASS
		camVec.clear();

		for (size_t i = 0; i < v.size(); ++i) {
			std::shared_ptr<nivision::USBCamera> camera =
			  std::make_shared<nivision::USBCamera>(v[i].c_str(), true);
			camera->SetBrightness(camBrightness);
			if (camExposure == CAM_AUTO)
				camera->SetExposureAuto();
			else
				camera->SetExposureManual(camExposure);
			if (camWhiteBalance == CAM_AUTO)
				camera->SetWhiteBalanceAuto();
			else
				camera->SetWhiteBalanceManual(camWhiteBalance);

			camera->OpenCamera();
			camVec.push_back(camera);
		}

		camServer->StartAutomaticCapture(camVec);
#else
		// defaults seem to be: Auto white bal; exposure 50%; brigthness 80%
		//
		// axis was: color level 70; brightness 50; sharpness 0; white balance Fixed Fluorescent 1; exposure Auto; no exposure priority
		std::vector<std::shared_ptr<USBCamera> > camVec;
		for (size_t i = 0; i < v.size(); ++i) {
			std::shared_ptr<USBCamera> camera =	std::make_shared<USBCamera>(v[i].c_str(), true);
			camera->OpenCamera();
			camera->SetExposureAuto();
			camera->SetBrightness(50);
			camera->SetWhiteBalanceManual(kFixedFluorescent1);
			camVec.push_back(camera);
		}
		camServer->StartAutomaticCapture(v);
#endif
		send_Debug_Double("thunderdashboard", "frontcamera", frontCam ? 1 : 0);
}

void Feedback::processCameraSettings()
{
	int n = (int)SmartDashboard::GetNumber("camera_brightness", camBrightness);
	if (n != camBrightness) {
		camBrightness = n;
		if (n > 100)
			n = 100;
		if (n < 0)
			n = 0;
		for (size_t i = 0; i < camVec.size(); ++i) {
			camVec[i]->SetBrightness(n);
		}
	}

	n = (int)SmartDashboard::GetNumber("camera_exposure", camExposure);
	if (n != camExposure) {
		camExposure = n;
		if (n > 100)
			n = 100;
		if (n < 0 && n != CAM_AUTO)
			n = 0;

		for (size_t i = 0; i < camVec.size(); ++i) {
			if (n == CAM_AUTO)
				camVec[i]->SetExposureAuto();
			else
				camVec[i]->SetExposureManual(n);
		}
	}

	std::string ns = SmartDashboard::GetString("camera_whitebalance", curWbSetting->name);
	if (ns.compare(curWbSetting->name) != 0) {
		for (size_t i = 0; i < 6; ++i) {
			if (ns.compare(wbSettings[i].name) == 0) {
				curWbSetting = wbSettings + i;

				for (size_t i = 0; i < camVec.size(); ++i) {
					if (curWbSetting->value == CAM_AUTO)
						camVec[i]->SetWhiteBalanceAuto();
					else
						camVec[i]->SetWhiteBalanceManual(curWbSetting->value);
				}
			}
		}
	}

}

void Feedback::send_Debug_String(const char *subsystemName, const char *dataName, const char *format, ...) {//

	std::string keyName;
		keyName = subsystemName;
		keyName += "_";
		keyName += dataName; //adds a _ after your subsytem name, the name of your specific value after that
	  char buffer[256]; //sets the maximum character count to 256
	  va_list args; // args is the "..."
	  va_start (args, format); //marks the start of the list of arguments(args)
	  vsnprintf (buffer,256,format, args); //printing the args after the keyName
	  va_end (args);//marks end of arguments (this allows us to change the arguments and not the keyname)
	  SmartDashboard::PutString(keyName.c_str(), buffer);//this prints keyName into the table, then the arguments
	}

void Feedback::send_Debug_Double(const char *subsystemName, const char *dataName, double format){

	std::string keyName;
	keyName = subsystemName;
	keyName += "_";
	keyName += dataName; // a _ after the subsystem name, and adds the name of the value after that
	SmartDashboard::PutNumber(keyName.c_str(), format); //this prints keyName and the data into the table
}

void Feedback::send_Editable_String(const char *subsystemName, const char *dataName, const char *format, ...) {//

	std::string keyName;
		keyName = subsystemName;
		keyName += "_";
		keyName += dataName; //adds a _ after your subsytem name, the name of your specific value after that
		keyName += "_e"; // End in _e to make editable
	  char buffer[256]; //sets the maximum character count to 256
	  va_list args; // args is the "..."
	  va_start (args, format); //marks the start of the list of arguments(args)
	  vsnprintf (buffer,256,format, args); //printing the args after the keyName
	  va_end (args);//marks end of arguments (this allows us to change the arguments and not the keyname)
	  SmartDashboard::PutString(keyName.c_str(), buffer);//this prints keyName into the table, then the arguments
	}

void Feedback::send_Editable_Double(const char *subsystemName, const char *dataName, double format){

	std::string keyName;
	keyName = subsystemName;
	keyName += "_";
	keyName += dataName; // a _ after the subsystem name, and adds the name of the value after that
	keyName += "_e"; // End in _e to make editable
	SmartDashboard::PutNumber(keyName.c_str(), format); //this prints keyName and the data into the table
}


std::string Feedback::get_Editable_String(const char *subsystemName, const char *dataName, llvm::StringRef defaultVal) {//

	std::string keyName;
		keyName = subsystemName;
		keyName += "_";
		keyName += dataName; //adds a _ after your subsytem name, the name of your specific value after that
		keyName += "_e"; // End in _e to make editable
	  return SmartDashboard::GetString(keyName.c_str(), defaultVal);//this prints keyName into the table, then the arguments
	}

double Feedback::get_Editable_Double(const char *subsystemName, const char *dataName, double defaultVal){

	std::string keyName;
	keyName = subsystemName;
	keyName += "_";
	keyName += dataName; // a _ after the subsystem name, and adds the name of the value after that
	keyName += "_e"; // End in _e to make editable
	return SmartDashboard::GetNumber(keyName.c_str(), defaultVal);
}



void Feedback::swapCamera()
{
#ifdef USE_TWO_CAMERAS
	if (hasBackCam) {
        frontCam = !frontCam;
        camServer->SetCurrentCameraIndex(frontCam ? 0 : 1);
	}
#endif
	send_Debug_Double("thunderdashboard", "frontcamera", frontCam ? 1 : 0);
}

bool Feedback::isFrontCamera()
{
	return (frontCam);
}


