#include "WPILib.h"
#include "Feedback.h"
#include "FuelSystem.h"

const double kIndexerPulseOnTime = .175;
const double kIndexerPulseOffTime = 0;


FuelSystem::FuelSystem(Bucket *bucket, Intake *intake, Shooter *shooter) :
	_pulseIsOn(false),
	_pulseOnTime(kIndexerPulseOnTime),
	_pulseOffTime(kIndexerPulseOffTime)

{
	_bucket = bucket;
	_intake = intake;
	_shooter = shooter;
	_action = STOP;
	_pulseTimer.Start();
}


void FuelSystem::SetAction (Action action)
{
	if (action == INTAKE)
	{
		_intake->Set_Direction(Intake::IN);
		_bucket->Beating_Bar_Set_Direction(Bucket::IN);
		_bucket->Start_Enqueue(Bucket::FORWARD);
		_bucket->Indexer_Set_Direction(Bucket::OUT);
		_shooter->End_Shoot();
	}

	if (action == OUTAKE){
		_intake->Set_Direction(Intake::OUT);
		_bucket->Beating_Bar_Set_Direction(Bucket::OUT);
		_bucket->Start_Enqueue(Bucket::BACKWARD);
		_bucket->Indexer_Set_Direction(Bucket::OUT);
		_shooter->End_Shoot();
	}
	if (action == SHOOT && _action != SHOOT){
		_intake->Set_Direction(Intake::STOP);
		_bucket->Beating_Bar_Set_Direction(Bucket::IN);
		_bucket->Start_Enqueue(Bucket::FORWARD);
		_bucket->Indexer_Set_Direction(Bucket::IN);
		_shooter->Start_Shoot();
		_pulseTimer.Reset();
		_pulseIsOn = true;
	}
	if (action == STOP){
		_intake->Set_Direction(Intake::STOP);
		_bucket->Beating_Bar_Set_Direction(Bucket::STOP);
		_bucket->Stop_Enqueue();
		_bucket->Indexer_Set_Direction(Bucket::STOP);
		_shooter->End_Shoot();
	}
	if (action == INTAKE_PREHEAT){
		_intake->Set_Direction(Intake::IN);
		_bucket->Beating_Bar_Set_Direction(Bucket::IN);
		_bucket->Start_Enqueue(Bucket::FORWARD);
		_bucket->Indexer_Set_Direction(Bucket::OUT);
		_shooter->Start_Shoot();
	}
	if (action == PREHEAT){
		_intake->Set_Direction(Intake::STOP);
		_bucket->Beating_Bar_Set_Direction(Bucket::STOP);
		_bucket->Stop_Enqueue();
		_bucket->Indexer_Set_Direction(Bucket::STOP);
		_shooter->Start_Shoot();
	}
	_action = action;
}
void FuelSystem::Process ()
{
	if (_action == SHOOT) {
		double d = _pulseTimer.Get();
		if (d < _pulseOnTime) {
			if (!_pulseIsOn) {
				_pulseIsOn = true;
				_bucket->Start_Enqueue(Bucket::FORWARD);
				_bucket->Indexer_Set_Direction(Bucket::IN);
		    }
		} else if (d < _pulseOffTime + _pulseOnTime) {
			if (_pulseIsOn) {
				_pulseIsOn = false;
				_bucket->Start_Enqueue(Bucket::OFF);
				_bucket->Indexer_Set_Direction(Bucket::STOP);
			}
		} else {
			_pulseTimer.Reset();
		}
	}
	_intake->Process();
	_bucket->Process();
	_shooter->Process();
}
void FuelSystem::Debug(Feedback *feedback){
	_intake->Debug(feedback);
	_bucket->Debug(feedback);
	_shooter->Debug(feedback);
}


void FuelSystem::Reset()
{
	_intake->Reset();
	_bucket->Reset();
	_shooter->Reset();
	_action = STOP;
}
void FuelSystem::Set_Bucket_Broken(bool Broken)
{
	_bucket->Set_Sensor_Broken(Broken);
}
void FuelSystem::Set_Intake_Broken(bool Broken)
{
	_intake->Set_Sensor_Broken(Broken);
}
void FuelSystem::Set_Shooter_Broken(bool Broken)
{
	_shooter->Shooter_Set_Broken(Broken);
}
void FuelSystem::Get_Dashboard_Values(Feedback *feedback)
{
	_intake->Get_Dashboard_Values(feedback);
	_bucket->Get_Dashboard_Values(feedback);
	_shooter->Get_Dashboard_Values(feedback);
	_pulseOnTime = feedback->get_Editable_Double("fuel", "pulse on time", _pulseOnTime);
	_pulseOffTime = feedback->get_Editable_Double("fuel", "pulse off time", _pulseOffTime);
}

void FuelSystem::Init_Dashboard_Values(Feedback *feedback)
{
	feedback->send_Editable_Double("fuel", "pulse on time", _pulseOnTime);
	feedback->send_Editable_Double("fuel", "pulse off time", _pulseOffTime);
	_intake->Init_Dashboard_Values(feedback);
	_bucket->Init_Dashboard_Values(feedback);
	_shooter->Init_Dashboard_Values(feedback);
}




