#include "WPILib.h"
#include "Gear.h"
#include "IOMap.h"

#ifdef ROBOTTWO
const double kRFlipperRetract = 0;
const double kLFlipperRetract= 1.0;
const double kRFlipperExtend = 1.0;
const double kLFlipperExtend = 0;
#else
const double kRFlipperRetract = 0.0;
const double kLFlipperRetract= 1.0;
const double kRFlipperExtend = 1.0;
const double kLFlipperExtend = 0;
#endif

Gear::Gear() :
SpikeSensor(DIG_IN_GEAR_SPIKE_SENSOR),
FlipperAngleL(PWM_OUT_FLIPPER_ANGLEL),
FlipperAngleR(PWM_OUT_FLIPPER_ANGLER),
_spikeSensorBroken(false),
_retracted(true),
_rFlipperRetract(kRFlipperRetract),
_lFlipperRetract(kLFlipperRetract),
_rFlipperExtend(kRFlipperExtend),
_lFlipperExtend(kLFlipperExtend)

{
}

void Gear::Process(){
}

void Gear::Extend_Flippers(){
	/*if (SpikeSensor.Get() && !_spikeSensorBroken){
		FlipperAngleL.Set(_lFlipperExtend);
		FlipperAngleR.Set(_rFlipperExtend);
		_retracted = false;
	}
	else if (!_spikeSensorBroken){
		FlipperAngleL.Set(_lFlipperRetract);
		FlipperAngleR.Set(_rFlipperRetract);
		_retracted = true;
	}
	else{*/
		FlipperAngleR.Set(_rFlipperExtend);
		FlipperAngleL.Set(_lFlipperExtend);
		_retracted = false;
	//}
	//if spike is present, open flippers to deploy gear
}

void Gear::Retract_Flippers(){
	_retracted = true;
	//if (GearSensor.Get()==false)
	FlipperAngleL.Set(_lFlipperRetract);
	FlipperAngleR.Set(_rFlipperRetract);
}

bool Gear::Is_Spike_Engaged(){
	if(_spikeSensorBroken == false)
	{
		bool currentState;
		currentState = SpikeSensor.Get();
		return currentState;
	}
	else
	{
		return false;
	}

}

bool Gear::Is_Gear_Present(){
	bool currentState = true;
	return currentState;
}

void Gear::Set_Spike_Sensor_Broken(bool SpikeSensorBroken){
	_spikeSensorBroken = SpikeSensorBroken;


}

void Gear::Reset(){
	Gear::Retract_Flippers();
}

void Gear::Init_Dashboard_Values(Feedback *feedback){
	feedback->send_Editable_Double("gear", "servo left retract", _lFlipperRetract);
	feedback->send_Editable_Double("gear", "servo left extend", _lFlipperExtend);
	feedback->send_Editable_Double("gear", "servo right retract", _rFlipperRetract);
	feedback->send_Editable_Double("gear", "servo right extend", _rFlipperExtend);
}

void Gear::Debug(Feedback *feedback){
	feedback->send_Debug_Double("gear", "Servo Left", FlipperAngleL.Get());
	feedback->send_Debug_Double("gear", "Servo Right", FlipperAngleR.Get());
	feedback->send_Debug_String("gear", "Spike Sensor Broken", _spikeSensorBroken ? "true":"false");
	feedback->send_Debug_String("gear", "mechanism is...", _retracted ? "retracted":"open");
}
void Gear::Get_Dashboard_Values(Feedback *feedback) {
	_lFlipperRetract = feedback->get_Editable_Double("gear", "servo left retract", _lFlipperRetract);
	_lFlipperExtend = feedback->get_Editable_Double("gear", "servo left extend", _lFlipperExtend);
	_rFlipperRetract = feedback->get_Editable_Double("gear", "servo right retract", _rFlipperRetract);
	_rFlipperExtend = feedback->get_Editable_Double("gear", "servo right extend", _rFlipperExtend);
}
