#include "Intake.h"
#include "IOMap.h"
#include "WPILib.h"

const double kIntakeSpeed = .375;

Intake::Intake():
	IntakeMotor(CAN_ID_INTAKE),
	IntakeEncoder(DIG_IN_INTAKE_ENCODER),
	Direction(IN),
	isBroken(false),
	_intakeSpeed(kIntakeSpeed)

{
}

void Intake::Set_Direction(IntakeDirection direction)
{
	Direction = direction;

}

void Intake::Set_Sensor_Broken(bool Broken){
	isBroken=Broken;
}

void Intake::Process(){
	switch(Direction){
		case IN:
			IntakeMotor.Set(_intakeSpeed);
		break;
		case OUT:
			IntakeMotor.Set(-_intakeSpeed);
		break;
		case STOP:
			IntakeMotor.Set(0);
		break;
	}

}

void Intake::Reset(){
	IntakeEncoder.Reset();
	IntakeMotor.Set(0);
	Direction = STOP;
}


void Intake::Debug(Feedback *feedback){
	feedback->send_Debug_Double("intake", "Current Draw", IntakeMotor.GetOutputCurrent());
	feedback->send_Debug_Double("intake", "Encoder count", IntakeEncoder.Get());
	feedback->send_Debug_Double("intake", "Direction", Direction);
}
void Intake::Init_Dashboard_Values(Feedback *feedback){
	feedback->send_Editable_Double("intake", "speed", _intakeSpeed);
}
void Intake::Get_Dashboard_Values(Feedback *feedback) {
	_intakeSpeed = feedback->get_Editable_Double("intake", "speed", _intakeSpeed);
}
