#include "Shooter.h"
#include "wpilib.h"
#include "IOMap.h"
#include <stdio.h>
#include <stdarg.h>
#include <vector>	//136.5
#include <string>


#define USE_UDP_LOGGER

// p = 0.02, 0.001 = i f = 0.04

// p .25, i 1.5 *10^-5, d 12, f .0325

const double kShootTolerancePercent = 0.01;
const double kRotationPerSec = 52.1;//62;
const double kAutoRotationPerSec = 30;
const double kShooterBrokenSpeed = .825;

#ifndef ROBOTTWO
const double kPVal = 0.041;
const double kIVal = 0;
const double kDVal = 5;
const double kFVal = .036;

const double kPAuto = 0;
const double kIAuto = 0;
const double kDAuto = 0;
const double kFAuto = .03;

#else
const double kPVal = 0.4;//0.05;
const double kIVal = 0;//0;
const double kDVal = 14;//5;
const double kFVal = .1487;//.034;
#endif

	Shooter::Shooter():
			_lastAutoShoot(true),
			_autoShoot(false),
			shooterMotor1(CAN_ID_SHOOTER_MAIN),
			shooterMotor2(CAN_ID_SHOOTER_SECONDARY),
			_shooting(false),
			_shootSpeed(0),
			_broken(false),
			_lastBroken(false),
			_brokenSpeed(kShooterBrokenSpeed),
			_pVal(kPVal),
			_iVal(kIVal),
			_dVal(kDVal),
			_fVal(kFVal),
			_shootingLast(false),
			timer(),
			_desiredRPS(kRotationPerSec),
			_desiredRPM(kRotationPerSec * 60.0),
			_goingUp(true),
			logger(NULL),
			loggerTimer()



	{
		shooterMotor2.SetControlMode(CANTalon::kFollower);
		shooterMotor2.Set(CAN_ID_SHOOTER_MAIN);
#ifdef ROBOTTWO
		shooterMotor1.SetFeedbackDevice(CANTalon::QuadEncoder);
		shooterMotor1.ConfigEncoderCodesPerRev(256);
#else
		shooterMotor1.SetFeedbackDevice(CANTalon::CtreMagEncoder_Relative);
		shooterMotor1.SetSensorDirection(true);

#endif
		shooterMotor1.SetControlMode(CANSpeedController::kSpeed);
		shooterMotor1.ConfigPeakOutputVoltage(12,0);
		shooterMotor1.ConfigNominalOutputVoltage(0,0);
		shooterMotor1.ConfigNeutralMode(CANTalon::kNeutralMode_Coast);
		shooterMotor2.ConfigNeutralMode(CANTalon::kNeutralMode_Coast);
		shooterMotor1.SetPID(_pVal, _iVal, _dVal,_fVal);
		shooterMotor1.Set(0);
		//shooterMotor1.SetNominalClosedLoopVoltage(12.2);
		//shooterMotor1.SetStatusFrameRateMs()
//		shooterMotor1.SetIzone(450);
		shooterMotor1.SetAllowableClosedLoopErr(20);
		timer.Start();
#ifdef USE_UDP_LOGGER
		logger = new UDPLogger(false);
		loggerTimer.Start();
#endif
	}
	void Shooter::Start_Shoot(){
		_shooting = true;
	}

	void Shooter::End_Shoot(){
		_shooting = false;
	}

	bool Shooter::Is_Shooter_At_Speed() {
		if (_broken)
			return true;

		double err = shooterMotor1.GetClosedLoopError(); // talon native units
		err /= 0.1; // Scale up to seconds
		err /= 256; // Div "ticks" to revs
		if (err < kShootTolerancePercent * _desiredRPS)
			return true;
		else
			return false;
	}

	void Shooter::Process(){
		if(timer.Get() > .1){
		//	printf("%d %f \n", shooterMotor1.GetClosedLoopError(), shooterMotor1.GetOutputVoltage());
			timer.Reset();
		}
		if(_autoShoot != _lastAutoShoot){
			if(_autoShoot == true){
		/*		_pVal = kPAuto;
				_iVal = kIAuto;
				_dVal = kDAuto;
				_fVal = kFAuto;
		*/		//_desiredRPM = kAutoRotationPerSec*60;
			}
			else{
		/*		_pVal = kPVal;
				_iVal = kIVal;
				_dVal = kDVal;
				_fVal = kFVal;*/
			//	_desiredRPM = kRotationPerSec*60;
			}
		}
		if(!_broken){
			if(_lastBroken != _broken){
				shooterMotor1.SetControlMode(CANSpeedController::kSpeed);
			}
			if (_shooting){
				shooterMotor1.Set(_desiredRPM);
#ifdef USE_UDP_LOGGER
			double v = shooterMotor1.GetSpeed();
			v /= 60.0;
			logger->log("%f,%d,%f,%f", loggerTimer.Get(), shooterMotor1.GetClosedLoopError(), v,
			shooterMotor1.GetOutputVoltage());

#endif
			}
			else {
#ifdef USE_UDP_LOGGER
				loggerTimer.Reset();
#endif
				shooterMotor1.Set(0);
				shooterMotor1.ClearIaccum();
			}
		}
		else{
			if(_lastBroken != _broken)
				shooterMotor1.SetControlMode(CANSpeedController::kPercentVbus);
			if (_shooting){
					shooterMotor1.Set(_brokenSpeed);
				/*	if(_brokenSpeed < 1 && _goingUp)
						_brokenSpeed = _brokenSpeed + .001;
					else if(_brokenSpeed == 1)
						_goingUp = false;
					else if(_brokenSpeed > 0 && !_goingUp)
						_brokenSpeed = _brokenSpeed - .001;*/

#ifdef USE_UDP_LOGGER
			double v = shooterMotor1.GetSpeed();
			v /= 60.0;
			logger->log("%f,%d,%f,%f", loggerTimer.Get(), shooterMotor1.GetClosedLoopError(), v,
		    shooterMotor1.GetOutputVoltage());
#endif

			}
			else {
				shooterMotor1.Set(0);
#ifdef USE_UDP_LOGGER
				loggerTimer.Reset();
#endif
			}
		}
		_lastBroken = _broken;
		_shootingLast = _shooting;

	}

	void Shooter::Shooter_Set_Broken(bool broken){
		if(broken == true){
			_broken = true;
		}
		else{
			_broken = false;
		}
	}


	void Shooter::Debug(Feedback *feedback){
		feedback->send_Debug_Double("shooter", "shooter speed", shooterMotor1.GetSpeed()); //should be in rpm i think
		feedback->send_Debug_Double("shooter", "PID Error", (double)shooterMotor1.GetClosedLoopError());
		feedback->send_Debug_String("shooter", "Encoder broken?", _broken ? "true":"false");
		feedback->send_Debug_Double("shooter", "current draw main", shooterMotor1.GetOutputCurrent());
		feedback->send_Debug_Double("shooter", "current draw follower", shooterMotor2.GetOutputCurrent());
		feedback->send_Debug_Double("shooter", "Encoder Count", shooterMotor1.GetEncPosition());
		feedback->send_Debug_Double("shooter", "Voltage", shooterMotor1.GetOutputVoltage());
		feedback->send_Debug_Double("shooter", "PID P", _pVal);
		feedback->send_Debug_Double("shooter", "PID I", _iVal);
		feedback->send_Debug_Double("shooter", "PID D", _dVal);
		feedback->send_Debug_Double("shooter", "PID F", _fVal);
		feedback->send_Debug_Double("shooter", "brokenSpeed", _brokenSpeed);
		feedback->send_Debug_Double("shooter", "desiredSpeed", _desiredRPS);

		double v = shooterMotor1.GetSpeed();
		if(_shooting){
			feedback->send_Debug_String("shooter", "chart_plot","%f,%d,%f,%f", loggerTimer.Get(), shooterMotor1.GetClosedLoopError(), v,
							shooterMotor1.GetOutputVoltage());
		}

		CANTalon::FeedbackDeviceStatus status = shooterMotor1.IsSensorPresent(CANTalon::CtreMagEncoder_Relative);
		bool sensorOk;
		switch(status)
		{
			case CANTalon::FeedbackStatusPresent:
				sensorOk = true;
				break;
			case CANTalon::FeedbackStatusNotPresent:
				sensorOk = false;
				break;
			case CANTalon::FeedbackStatusUnknown:
				sensorOk = false;
				break;
		}
		feedback->send_Debug_String("thunderdashboard", "yellow", sensorOk ? "0" : "1");

	}

	void Shooter::Reset(){
		shooterMotor1.Set(0);
		shooterMotor1.ClearIaccum();
		_shooting = false;
		_broken = false;
	}

	void Shooter::Init_Dashboard_Values(Feedback *feedback){
		feedback->send_Editable_Double("shooter", "PID P", _pVal);
		feedback->send_Editable_Double("shooter", "PID I", _iVal);
		feedback->send_Editable_Double("shooter", "PID D", _dVal);
		feedback->send_Editable_Double("shooter", "PID F", _fVal);
		feedback->send_Editable_Double("shooter", "brokenSpeed", _brokenSpeed);
		feedback->send_Editable_Double("shooter", "desiredSpeed", _desiredRPS);
		feedback->send_Debug_String("shooter", "chart_plotmin", "0, -4800, 0, 0");
		feedback->send_Debug_String("shooter", "chart_plotmax", "0, 4800, 70, 13");
		feedback->send_Debug_String("shooter", "chart_plotlab", "time, error, RPS, voltage");
		feedback->send_Debug_String("shooter", "chart_plotclr", "0, 1, 0, 0");

#ifdef USE_UDP_LOGGER
		llvm::StringRef ipref = SmartDashboard::GetString("thunderdashboard_ip", "10.17.15.5");
		printf("Shooter logging to %s\n", ipref.str().c_str());
		logger->init(ipref.str().c_str(), 1511);
#endif

	}

	void Shooter::Get_Dashboard_Values(Feedback *feedback){
		double p, i, d, f;

		p = feedback->get_Editable_Double("shooter", "PID P", _pVal);
		i = feedback->get_Editable_Double("shooter", "PID I", _iVal);
		d = feedback->get_Editable_Double("shooter", "PID D", _dVal);
		f = feedback->get_Editable_Double("shooter", "PID F", _fVal);


	//	_brokenSpeed = feedback->get_Editable_Double("shooter", "brokenSpeed", _brokenSpeed);
		_desiredRPS =  feedback->get_Editable_Double("shooter", "desiredSpeed", _desiredRPS);
		_desiredRPM = _desiredRPS * 60.0;
		if (p != _pVal || d != _dVal || i != _iVal || f != _fVal) {
			_pVal = p;
			_dVal = d;
			_iVal = i;
			_fVal = f;
			shooterMotor1.SetPID(p, i, d, f);
		}
	}


