#ifndef TCAMSERV_H
#define TCAMSERV_H

#include "Vision/USBCamera.h"
#include "ErrorBase.h"
#include "Vision/nivision.h"
#include "Vision/NIIMAQdx.h"

#include "HAL/cpp/priority_mutex.h"
#include <thread>
#include <memory>
#include <condition_variable>
#include <tuple>
#include <vector>

class ThunderCameraServer : public ErrorBase {
 private:
  static constexpr uint16_t kPort = 1180;
  static constexpr uint8_t kMagicNumber[] = {0x01, 0x00, 0x00, 0x00};
  static constexpr uint32_t kSize640x480 = 0;
  static constexpr uint32_t kSize320x240 = 1;
  static constexpr uint32_t kSize160x120 = 2;
  static constexpr int32_t kHardwareCompression = -1;
  static constexpr uint32_t kMaxImageSize = 200000;
  static constexpr int kInvalidCamIndex = -1;

 protected:
  ThunderCameraServer();

  std::vector<std::shared_ptr<nivision::USBCamera> > m_cameras;
  int m_curCamIndex;
  std::thread m_serverThread;
  std::thread m_captureThread;
  priority_recursive_mutex m_imageMutex;
  std::condition_variable_any m_newImageVariable;
  std::vector<uint8_t*> m_dataPool;
  unsigned int m_quality;
  bool m_autoCaptureStarted;
  bool m_autoCaptureStop;
  bool m_hwClient;
  std::tuple<uint8_t*, unsigned int, unsigned int, bool> m_imageData;

  void Serve();
  void AutoCapture();
  void SetImageData(uint8_t* data, unsigned int size, unsigned int start = 0,
                    bool imaqData = false);
  void FreeImageData(
      std::tuple<uint8_t*, unsigned int, unsigned int, bool> imageData);
  void SetSize(unsigned int size);

  struct Request {
    uint32_t fps;
    int32_t compression;
    uint32_t size;
  };

 public:
  static ThunderCameraServer* GetInstance();
  void SetImage(Image const* image);

  void StopAutomaticCapture();
  void StartAutomaticCapture(
      char const* cameraName = nivision::USBCamera::kDefaultCameraName);
  void StartAutomaticCapture(std::vector<std::string> cameraNames);

  void SetCurrentCameraIndex(int n);

  /**
   * Start automatically capturing images to send to the dashboard.
   *
   * You should call this method to just see a camera feed on the
   * dashboard without doing any vision processing on the roboRIO.
   * {@link #SetImage} should not be called after this is called.
   *
   * @param camera The camera interface (eg. USBCamera)
   */
  void StartAutomaticCapture(std::shared_ptr<nivision::USBCamera> camera);
  void StartAutomaticCapture(std::vector<std::shared_ptr<nivision::USBCamera> > cameras);

  bool IsAutoCaptureStarted();

  void SetQuality(unsigned int quality);
  unsigned int GetQuality();

};

#endif
