#include "UDPLogger.h"
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#define ENABLE_UDP_LOGGER

UDPLogger::UDPLogger(bool print) :
	fd(0), stdoutprint(print), enabled(true) {
}

UDPLogger::~UDPLogger() {
	if (fd)
		close(fd);
}

void UDPLogger::init(const char *ip, int port) {
#ifdef ENABLE_UDP_LOGGER
	memset(&saddr, 0, sizeof(saddr));
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(port);

	if (inet_aton(ip, &saddr.sin_addr) == -1) {
		printf("UDPLogger: Bad inet_aton\n");
		return;
	}

	int tmpfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (tmpfd == -1) {
		printf("UDPLogger: Bad socket\n");
		return;
	}
	fd = tmpfd;
#endif
}

void UDPLogger::log(const char *fmt, ...) {
	if (enabled) {
		if (fd) {
			va_list args;
			va_start(args, fmt);
			vsprintf(buf, fmt, args);
			va_end(args);
			if (sendto(fd, buf, strlen(buf), 0, (sockaddr *) &saddr,
					sizeof(saddr)) == -1) {
#if !SUPPRESS_ERRORS
				printf("UDPLogger: sendto failed\n");
#endif
			}
		}
		if (stdoutprint) {
			va_list args;
			va_start(args, fmt);
			vprintf(fmt, args);
			va_end(args);
		}
	}
}

void UDPLogger::setEnabled(bool en) {
	enabled = en;
}
