#include "WPILib.h"

#include "Drive.h"
#include "Controls.h"
#include "Shooter.h"
#include "Autonomoose.h"
#include "Bucket.h"
#include "Climbing.h"
#include "Gear.h"
#include "Feedback.h"
#include "Lights.h"
#include "FuelSystem.h"

class ThunderBot : public SampleRobot
{

private:
		Drive drive;
		Bucket bucket;
		Intake intake;
		Gear gear;
		Shooter shooter;
		FuelSystem fuelSystem;
		Climbing climbing;
		Feedback feedback;
		Lights lights;
		Autonomoose autonomoose;
		Controls controls;

	const double MAIN_LOOP_WAIT_TIME = 0.05;
//
public:
	ThunderBot():drive(), bucket(), intake(), gear(), shooter(), fuelSystem(&bucket, &intake, &shooter), climbing(), lights(), autonomoose(&drive, &intake, &bucket, &gear, &shooter, &fuelSystem), controls(&drive, &gear, &fuelSystem, &feedback, &climbing)
	{
		//Note SmartDashboard is not initialized here, wait until RobotInit to make SmartDashboard calls
	}

	void RobotInit()
	{
		/*
		chooser = new SendableChooser();
		chooser->AddDefault(autoNameDefault, (void*)&autoNameDefault);
		chooser->AddObject(autoNameCustom, (void*)&autoNameCustom);
		SmartDashboard::PutData("Auto Modes", chooser);
		*/
		feedback.initCameras();
		fuelSystem.Init_Dashboard_Values(&feedback);
		gear.Init_Dashboard_Values(&feedback);
		climbing.Init_Dashboard_Values(&feedback);
		autonomoose.AutoDashboardSend();
	}

	void Disabled()
	{

		while (IsDisabled()) {

			feedback.processCameraSettings();

			controls.Process(Controls::DISABLED);

			lights.process(IsEnabled(), IsAutonomous());

			drive.Debug(&feedback);
			fuelSystem.Debug(&feedback);
			gear.Debug(&feedback);
			climbing.Debug(&feedback);

			Wait(MAIN_LOOP_WAIT_TIME);
		}
	}

	void Autonomous()
	{
		/*
		std::string autoSelected = *((std::string*)chooser->GetSelected());
		//std::string autoSelected = SmartDashboard::GetString("Auto Selector", autoNameDefault);
		std::cout << "Auto selected: " << autoSelected << std::endl;
		*/
		autonomoose.Reset();

		double autoMode = SmartDashboard::GetNumber("Auto_Mode", 0);	// use default auto mode if not set
		autonomoose.Set_Auto_Mode((Autonomoose::AutoMode)autoMode);



		drive.Reset();
		fuelSystem.Reset();
		gear.Reset();
		climbing.Reset();

		while (IsAutonomous() && IsEnabled())
		{
			feedback.processCameraSettings();

			drive.Debug(&feedback);
			fuelSystem.Debug(&feedback);
			gear.Debug(&feedback);
			climbing.Debug(&feedback);



			lights.process(IsEnabled(), IsAutonomous());

			autonomoose.Process();
			drive.Process();
			fuelSystem.Process();
			gear.Process();
			climbing.Process();

			Wait(MAIN_LOOP_WAIT_TIME);
		}
	}

	/**
	 * Runs the motors with arcade steering.
	 */
	void OperatorControl()
	{
		gear.Reset();
		autonomoose.Reset();
		drive.Reset();
		fuelSystem.Reset();
		climbing.Reset();


		while (IsOperatorControl() && IsEnabled())
		{

			drive.Debug(&feedback);
			fuelSystem.Debug(&feedback);
			gear.Debug(&feedback);
			climbing.Debug(&feedback);

			lights.process(IsEnabled(), IsAutonomous());
			fuelSystem.Process();
			gear.Process();
			climbing.Process();
			controls.Process(Controls::TELE_OP);
			drive.Process();

			Wait(MAIN_LOOP_WAIT_TIME);// wait for a motor update time

		}
	}

	/**
	 * Runs during test mode
	 */
	void Test()
	{
		while (IsTest() && IsEnabled()) {
			Wait(MAIN_LOOP_WAIT_TIME);
		}
	}
};

START_ROBOT_CLASS(ThunderBot)
